// Copyright (c) 2020, the Dart project authors.  Please see the AUTHORS file
// for details. All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

// Struct sizes are a mapping of name to 32-bit and 64-bit sizes. The easiest
// way to generate the first two maps is by compiling and running
// struct_sizes.cpp on both 32-bit and 64-bit architectures, and copying the
// resultant output directly into this file.

const structSize64 = {
  'BLUETOOTH_ADDRESS': 8,
  'BLUETOOTH_AUTHENTICATION_CALLBACK_PARAMS': 576,
  'BLUETOOTH_PIN_INFO': 17,
  'CHAR_INFO': 4,
  'COR_FIELD_OFFSET': 8,
  'DEVMODE': 220,
  'GUID': 16,
  'INPUT': 40,
  'MMTIME': 12,
  'NOTIFYICONDATA': 976,
  'OVERLAPPED': 32,
  'PROCESS_HEAP_ENTRY': 40,
  'PROPVARIANT': 24,
  'SHELLEXECUTEINFO': 112,
  'STARTUPINFOEX': 112,
  'SYSTEM_INFO': 48,
  'TASKDIALOGCONFIG': 160,
  'TYPEDESC': 16,
  'VARIANT': 24,
  'ACCEL': 6,
  'ACL': 8,
  'ACTCTX': 56,
  'APPX_PACKAGE_SETTINGS': 16,
  'BIND_OPTS': 16,
  'BITMAP': 32,
  'BITMAPFILEHEADER': 14,
  'BITMAPINFO': 44,
  'BITMAPINFOHEADER': 40,
  'BLENDFUNCTION': 4,
  'BLUETOOTH_DEVICE_INFO': 560,
  'BLUETOOTH_DEVICE_SEARCH_PARAMS': 40,
  'BLUETOOTH_FIND_RADIO_PARAMS': 4,
  'BLUETOOTH_OOB_DATA_INFO': 32,
  'BLUETOOTH_RADIO_INFO': 520,
  'BSMINFO': 32,
  'CHANGEFILTERSTRUCT': 8,
  'CHOOSECOLOR': 72,
  'CHOOSEFONT': 104,
  'COLORADJUSTMENT': 24,
  'COMDLG_FILTERSPEC': 16,
  'CONSOLE_CURSOR_INFO': 8,
  'CONSOLE_SCREEN_BUFFER_INFO': 22,
  'CONSOLE_SELECTION_INFO': 16,
  'COORD': 4,
  'CREATESTRUCT': 80,
  'CREDENTIAL': 80,
  'CREDENTIAL_ATTRIBUTE': 24,
  'CURSORINFO': 24,
  'DESIGNVECTOR': 72,
  'DIBSECTION': 104,
  'DISPPARAMS': 24,
  'DLGITEMTEMPLATE': 18,
  'DLGTEMPLATE': 18,
  'DLLVERSIONINFO': 20,
  'DRAWTEXTPARAMS': 20,
  'ENUMLOGFONTEX': 348,
  'EXCEPINFO': 64,
  'FILETIME': 8,
  'FINDREPLACE': 80,
  'FUNCDESC': 88,
  'GESTURECONFIG': 12,
  'GESTUREINFO': 56,
  'GESTURENOTIFYSTRUCT': 24,
  'GUITHREADINFO': 72,
  'ICONINFO': 32,
  'ICONINFOEX': 1080,
  'IDLDESC': 16,
  'INITCOMMONCONTROLSEX': 8,
  'ITEMIDLIST': 3,
  'KNOWNFOLDER_DEFINITION': 112,
  'LASTINPUTINFO': 8,
  'LOGBRUSH': 16,
  'LOGFONT': 92,
  'LOGPALETTE': 8,
  'LUID': 8,
  'MCI_OPEN_PARMS': 36,
  'MCI_PLAY_PARMS': 16,
  'MCI_SEEK_PARMS': 12,
  'MCI_STATUS_PARMS': 24,
  'MENUINFO': 40,
  'MENUITEMINFO': 80,
  'MENUITEMTEMPLATE': 6,
  'MENUITEMTEMPLATEHEADER': 4,
  'METAFILEPICT': 24,
  'MIDIEVENT': 16,
  'MIDIHDR': 112,
  'MIDIINCAPS': 76,
  'MIDIOUTCAPS': 84,
  'MIDIPROPTEMPO': 8,
  'MIDIPROPTIMEDIV': 8,
  'MIDISTRMBUFFVER': 12,
  'MINMAXINFO': 40,
  'MODLOAD_DATA': 24,
  'MONITORINFO': 40,
  'MOUSEMOVEPOINT': 24,
  'MSG': 48,
  'NEWTEXTMETRIC': 76,
  'NLM_SIMULATED_PROFILE_INFO': 524,
  'NONCLIENTMETRICS': 504,
  'OPENFILENAME': 152,
  'OSVERSIONINFO': 276,
  'OSVERSIONINFOEXW': 284,
  'PAINTSTRUCT': 72,
  'PALETTEENTRY': 4,
  'PARAMDESC': 16,
  'PARAMDESCEX': 32,
  'PHYSICAL_MONITOR': 264,
  'POINT': 8,
  'POINTS': 4,
  'POLYTEXT': 56,
  'POWERBROADCAST_SETTING': 24,
  'PROCESS_INFORMATION': 24,
  'PROPERTYKEY': 20,
  'RECT': 16,
  'RGBQUAD': 4,
  'SAFEARRAY': 32,
  'SAFEARRAYBOUND': 8,
  'SCROLLBARINFO': 60,
  'SCROLLINFO': 28,
  'SECURITY_ATTRIBUTES': 24,
  'SECURITY_DESCRIPTOR': 40,
  'SHITEMID': 3,
  'SHQUERYRBINFO': 24,
  'SIZE': 8,
  'SMALL_RECT': 8,
  'SOLE_AUTHENTICATION_SERVICE': 24,
  'STARTUPINFO': 104,
  'STATSTG': 80,
  'STYLESTRUCT': 8,
  'SYMBOL_INFO': 88,
  'SYSTEM_BATTERY_STATE': 32,
  'SYSTEM_POWER_STATUS': 12,
  'SYSTEMTIME': 16,
  'TASKDIALOG_BUTTON': 12,
  'TEXTMETRIC': 60,
  'TITLEBARINFO': 44,
  'TITLEBARINFOEX': 140,
  'TOKEN_APPCONTAINER_INFORMATION': 8,
  'TOUCHINPUT': 48,
  'TPMPARAMS': 20,
  'TYPEATTR': 96,
  'UPDATELAYEREDWINDOWINFO': 80,
  'VALENT': 32,
  'VS_FIXEDFILEINFO': 52,
  'WAVEFORMATEX': 18,
  'WAVEHDR': 48,
  'WAVEOUTCAPS': 84,
  'WIN32_FIND_DATA': 592,
  'WINDOWINFO': 60,
  'WINDOWPLACEMENT': 44,
  'WNDCLASS': 72,
  'WNDCLASSEX': 80,
  'XFORM': 24,
};

const structSize32 = {
  'BLUETOOTH_ADDRESS': 8,
  'BLUETOOTH_AUTHENTICATION_CALLBACK_PARAMS': 576,
  'BLUETOOTH_PIN_INFO': 17,
  'CHAR_INFO': 4,
  'COR_FIELD_OFFSET': 8,
  'DEVMODE': 220,
  'GUID': 16,
  'INPUT': 28,
  'MMTIME': 12,
  'NOTIFYICONDATA': 956,
  'OVERLAPPED': 20,
  'PROCESS_HEAP_ENTRY': 28,
  'PROPVARIANT': 16,
  'SHELLEXECUTEINFO': 60,
  'STARTUPINFOEX': 72,
  'SYSTEM_INFO': 36,
  'TASKDIALOGCONFIG': 96,
  'TYPEDESC': 8,
  'VARIANT': 16,
  'ACCEL': 6,
  'ACL': 8,
  'ACTCTX': 32,
  'APPX_PACKAGE_SETTINGS': 8,
  'BIND_OPTS': 16,
  'BITMAP': 24,
  'BITMAPFILEHEADER': 14,
  'BITMAPINFO': 44,
  'BITMAPINFOHEADER': 40,
  'BLENDFUNCTION': 4,
  'BLUETOOTH_DEVICE_INFO': 560,
  'BLUETOOTH_DEVICE_SEARCH_PARAMS': 32,
  'BLUETOOTH_FIND_RADIO_PARAMS': 4,
  'BLUETOOTH_OOB_DATA_INFO': 32,
  'BLUETOOTH_RADIO_INFO': 520,
  'BSMINFO': 20,
  'CHANGEFILTERSTRUCT': 8,
  'CHOOSECOLOR': 36,
  'CHOOSEFONT': 60,
  'COLORADJUSTMENT': 24,
  'COMDLG_FILTERSPEC': 8,
  'CONSOLE_CURSOR_INFO': 8,
  'CONSOLE_SCREEN_BUFFER_INFO': 22,
  'CONSOLE_SELECTION_INFO': 16,
  'COORD': 4,
  'CREATESTRUCT': 48,
  'CREDENTIAL': 52,
  'CREDENTIAL_ATTRIBUTE': 16,
  'CURSORINFO': 20,
  'DESIGNVECTOR': 72,
  'DIBSECTION': 84,
  'DISPPARAMS': 16,
  'DLGITEMTEMPLATE': 18,
  'DLGTEMPLATE': 18,
  'DLLVERSIONINFO': 20,
  'DRAWTEXTPARAMS': 20,
  'ENUMLOGFONTEX': 348,
  'EXCEPINFO': 32,
  'FILETIME': 8,
  'FINDREPLACE': 40,
  'FUNCDESC': 52,
  'GESTURECONFIG': 12,
  'GESTUREINFO': 48,
  'GESTURENOTIFYSTRUCT': 20,
  'GUITHREADINFO': 48,
  'ICONINFO': 20,
  'ICONINFOEX': 1068,
  'IDLDESC': 8,
  'INITCOMMONCONTROLSEX': 8,
  'ITEMIDLIST': 3,
  'KNOWNFOLDER_DEFINITION': 76,
  'LASTINPUTINFO': 8,
  'LOGBRUSH': 12,
  'LOGFONT': 92,
  'LOGPALETTE': 8,
  'LUID': 8,
  'MCI_OPEN_PARMS': 20,
  'MCI_PLAY_PARMS': 12,
  'MCI_SEEK_PARMS': 8,
  'MCI_STATUS_PARMS': 16,
  'MENUINFO': 28,
  'MENUITEMINFO': 48,
  'MENUITEMTEMPLATE': 6,
  'MENUITEMTEMPLATEHEADER': 4,
  'METAFILEPICT': 16,
  'MIDIEVENT': 16,
  'MIDIHDR': 64,
  'MIDIINCAPS': 76,
  'MIDIOUTCAPS': 84,
  'MIDIPROPTEMPO': 8,
  'MIDIPROPTIMEDIV': 8,
  'MIDISTRMBUFFVER': 12,
  'MINMAXINFO': 40,
  'MODLOAD_DATA': 20,
  'MONITORINFO': 40,
  'MOUSEMOVEPOINT': 16,
  'MSG': 28,
  'NEWTEXTMETRIC': 76,
  'NLM_SIMULATED_PROFILE_INFO': 524,
  'NONCLIENTMETRICS': 504,
  'OPENFILENAME': 88,
  'OSVERSIONINFO': 276,
  'OSVERSIONINFOEXW': 284,
  'PAINTSTRUCT': 64,
  'PALETTEENTRY': 4,
  'PARAMDESC': 8,
  'PARAMDESCEX': 24,
  'PHYSICAL_MONITOR': 260,
  'POINT': 8,
  'POINTS': 4,
  'POLYTEXT': 40,
  'POWERBROADCAST_SETTING': 24,
  'PROCESS_INFORMATION': 16,
  'PROPERTYKEY': 20,
  'RECT': 16,
  'RGBQUAD': 4,
  'SAFEARRAY': 24,
  'SAFEARRAYBOUND': 8,
  'SCROLLBARINFO': 60,
  'SCROLLINFO': 28,
  'SECURITY_ATTRIBUTES': 12,
  'SECURITY_DESCRIPTOR': 20,
  'SHITEMID': 3,
  'SHQUERYRBINFO': 20,
  'SIZE': 8,
  'SMALL_RECT': 8,
  'SOLE_AUTHENTICATION_SERVICE': 16,
  'STARTUPINFO': 68,
  'STATSTG': 72,
  'STYLESTRUCT': 8,
  'SYMBOL_INFO': 88,
  'SYSTEM_BATTERY_STATE': 32,
  'SYSTEM_POWER_STATUS': 12,
  'SYSTEMTIME': 16,
  'TASKDIALOG_BUTTON': 8,
  'TEXTMETRIC': 60,
  'TITLEBARINFO': 44,
  'TITLEBARINFOEX': 140,
  'TOKEN_APPCONTAINER_INFORMATION': 4,
  'TOUCHINPUT': 40,
  'TPMPARAMS': 20,
  'TYPEATTR': 76,
  'UPDATELAYEREDWINDOWINFO': 40,
  'VALENT': 16,
  'VS_FIXEDFILEINFO': 52,
  'WAVEFORMATEX': 18,
  'WAVEHDR': 32,
  'WAVEOUTCAPS': 84,
  'WIN32_FIND_DATA': 592,
  'WINDOWINFO': 60,
  'WINDOWPLACEMENT': 44,
  'WNDCLASS': 40,
  'WNDCLASSEX': 48,
  'XFORM': 24,
};
